"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = _default;

var _helperModuleTransforms = require("@babel/helper-module-transforms");

// @ts-expect-error missing typedef
var IGNORED_MODULES = ['react'];

function _default(_ref) {
  var t = _ref.types;
  var affectedParents = new Map();

  var createRequireExpression = function createRequireExpression(programPath, info) {
    var requireExpression = t.callExpression(t.identifier('require'), [t.stringLiteral(info.source)]);

    if (info.property === null) {
      return (0, _helperModuleTransforms.wrapInterop)(programPath, requireExpression, 'namespace');
    }

    return t.memberExpression(info.property === 'default' ? (0, _helperModuleTransforms.wrapInterop)(programPath, requireExpression, 'default') : requireExpression, t.identifier(info.property));
  };

  var createConstRequireExpression = function createConstRequireExpression(programPath, info) {
    return t.variableDeclaration('const', [t.variableDeclarator(t.identifier(info.local), createRequireExpression(programPath, info))]);
  };

  var insertDeclaration = function insertDeclaration(path, programPath, info) {
    var _affectedParents$get;

    var parentWithBody = path.findParent(function (p) {
      var _p$node, _p$node$body, _p$node2;

      return ((_p$node = p.node) === null || _p$node === void 0 ? void 0 : (_p$node$body = _p$node.body) === null || _p$node$body === void 0 ? void 0 : _p$node$body.length) && ((_p$node2 = p.node) === null || _p$node2 === void 0 ? void 0 : _p$node2.type) !== 'ClassBody';
    });

    if (!affectedParents.has(info.local)) {
      affectedParents.set(info.local, new WeakSet());
    }

    if (!((_affectedParents$get = affectedParents.get(info.local)) === null || _affectedParents$get === void 0 ? void 0 : _affectedParents$get.has(parentWithBody)) && !parentWithBody.scope.hasBinding(info.local, true)) {
      var _affectedParents$get2;

      parentWithBody.unshiftContainer( // @ts-expect-error wrong typedef type
      'body', createConstRequireExpression(programPath, info));
      (_affectedParents$get2 = affectedParents.get(info.local)) === null || _affectedParents$get2 === void 0 ? void 0 : _affectedParents$get2.add(parentWithBody);
    }
  };

  return {
    inherits: require('babel-plugin-syntax-jsx'),
    visitor: {
      ImportDeclaration: function (_ImportDeclaration) {
        function ImportDeclaration(_x) {
          return _ImportDeclaration.apply(this, arguments);
        }

        ImportDeclaration.toString = function () {
          return _ImportDeclaration.toString();
        };

        return ImportDeclaration;
      }(function (path) {
        var specifiers = path.node.specifiers;
        var programPath = path.parentPath;

        if (IGNORED_MODULES.includes(path.node.source.value)) {
          return;
        }

        var hasReplaced = false;
        specifiers.forEach(function (specifier) {
          var info = specifier.type === 'ImportDefaultSpecifier' ? {
            local: specifier.local.name,
            property: 'default',
            source: path.node.source.value
          } : specifier.type === 'ImportSpecifier' ? {
            local: specifier.local.name,
            property: specifier.imported.name,
            source: path.node.source.value
          } : {
            local: specifier.local.name,
            property: null,
            source: path.node.source.value
          };
          var binding = path.scope.getBinding(info.local);

          if (!binding) {
            return;
          }

          path.scope.rename(info.local, "____________".concat(info.local, "____________"));
          binding.referencePaths.forEach(function (referencePath) {
            insertDeclaration(referencePath, programPath, info);
            hasReplaced = true;
          });
          path.scope.rename("____________".concat(info.local, "____________"), info.local);
        });
        if (hasReplaced) path.remove();
      })
    }
  };
}